@@ -1,3 +1,6 @@
+#ifndef _SEPOL_CONTEXT_INTERNAL_H_
+#define _SEPOL_CONTEXT_INTERNAL_H_
+
 #include <sepol/context_record.h>
 #include "dso.h"
 
@@ -13,3 +16,5 @@ hidden_proto(sepol_context_set_mls)
 hidden_proto(sepol_context_set_role)
 hidden_proto(sepol_context_set_type)
 hidden_proto(sepol_context_set_user)
+
+#endif
