@@ -1,3 +1,6 @@
+#ifndef _SEPOL_IFACE_INTERNAL_H_
+#define _SEPOL_IFACE_INTERNAL_H_
+
 #include <sepol/iface_record.h>
 #include <sepol/interfaces.h>
 #include "dso.h"
@@ -12,3 +15,5 @@ hidden_proto(sepol_iface_key_unpack)
 hidden_proto(sepol_iface_set_ifcon)
 hidden_proto(sepol_iface_set_msgcon)
 hidden_proto(sepol_iface_set_name)
+
+#endif
