@@ -1,3 +1,6 @@
+#ifndef _SEPOL_PORT_INTERNAL_H_
+#define _SEPOL_PORT_INTERNAL_H_
+
 #include <sepol/port_record.h>
 #include <sepol/ports.h>
 #include "dso.h"
@@ -14,3 +17,5 @@ hidden_proto(sepol_port_key_unpack)
 hidden_proto(sepol_port_set_con)
 hidden_proto(sepol_port_set_proto)
 hidden_proto(sepol_port_set_range)
+
+#endif
