@@ -49,7 +49,7 @@ int sepol_port_key_create(
 hidden_def(sepol_port_key_create)
 
 void sepol_port_key_unpack(
-	sepol_port_key_t* key,
+	const sepol_port_key_t* key,
 	int* low, int* high, int* proto) {
 
 	*low = key->low;
@@ -60,14 +60,14 @@ hidden_def(sepol_port_key_unpack)
 
 int sepol_port_key_extract(
 	sepol_handle_t* handle,	
-	sepol_port_t* port, 
+	const sepol_port_t* port, 
 	sepol_port_key_t** key_ptr) {
 
 	if (sepol_port_key_create(
 		handle, port->low, port->high, port->proto, key_ptr) < 0) {
 
 		ERR(handle, "could not extract key from port %s %d:%d", 
-			sepol_port_get_proto_str(port),
+			sepol_port_get_proto_str(port->proto),
 			port->low, port->high);
 
 		return STATUS_ERR;
@@ -76,53 +76,111 @@ int sepol_port_key_extract(
 	return STATUS_SUCCESS;
 }
 
-void sepol_port_key_free(sepol_port_key_t* key) {
+void sepol_port_key_free(
+	sepol_port_key_t* key) {
 	free(key);
 }
 
 int sepol_port_compare(
-	sepol_port_t* port, 
-	sepol_port_key_t* key) {
+	const sepol_port_t* port, 
+	const sepol_port_key_t* key) {
 
-	if ((port->low <= key->low) && 
-	    (port->high >= key->high) &&
+	if ((port->low == key->low) && 
+	    (port->high == key->high) &&
 	    (port->proto == key->proto))
 		return 0;
 
-	return 1;
+	if (port->low < key->low) 
+		return -1;
+
+	else if (key->low < port->low)
+		return 1;
+
+	else if (port->high < key->high)
+		return -1;
+
+	else if (key->high < port->high)
+		return 1;
+
+	else if (port->proto < key->proto)
+		return -1;
+	
+	else
+		return 1;
+}
+
+int sepol_port_compare2(
+	const sepol_port_t* port,
+	const sepol_port_t* port2) {
+
+	if ((port->low == port2->low) &&
+	    (port->high == port2->high) &&
+	    (port->proto == port2->proto))
+		return 0;
+
+	if (port->low < port2->low)
+		return -1;
+
+	else if (port2->low < port->low)
+		return 1;
+
+	else if (port->high < port2->high)
+		return -1;
+
+	else if (port2->high < port->high)
+		return 1;
+
+	else if (port->proto < port2->proto)
+		return -1;
+
+	else
+		return 1;
 }
 
 /* Port */
-int sepol_port_get_low(sepol_port_t* port) {
+int sepol_port_get_low(
+	const sepol_port_t* port) {
+
 	return port->low;
 }
 hidden_def(sepol_port_get_low)
 
-int sepol_port_get_high(sepol_port_t* port) {
+int sepol_port_get_high(
+	const sepol_port_t* port) {
+	
 	return port->high;
 }
 hidden_def(sepol_port_get_high)
 
-void sepol_port_set_port(sepol_port_t* port, int port_num) {
+void sepol_port_set_port(
+	sepol_port_t* port, 
+	int port_num) {
+
 	port->low = port_num;
 	port->high = port_num;
 }
 
-void sepol_port_set_range(sepol_port_t* port, int low, int high) {
+void sepol_port_set_range(
+	sepol_port_t* port, 
+	int low, int high) {
+
 	port->low = low;
 	port->high = high;
 }
 hidden_def(sepol_port_set_range)
 
 /* Protocol */
-int sepol_port_get_proto(sepol_port_t* port) {
+int sepol_port_get_proto(
+	const sepol_port_t* port) {
 
 	return port->proto;
 }
 hidden_def(sepol_port_get_proto)
 
-const char* sepol_port_get_proto_str(sepol_port_t* port) {
-	switch (port->proto) {
+const char* sepol_port_get_proto_str(
+	int proto) {
+
+	switch (proto) {
 		case SEPOL_PROTO_UDP:
 			return "udp";
 		case SEPOL_PROTO_TCP:
@@ -168,7 +226,7 @@ hidden_def(sepol_port_create)
 /* Deep copy clone */
 int sepol_port_clone(
 	sepol_handle_t* handle,
-	sepol_port_t* port, 
+	const sepol_port_t* port, 
 	sepol_port_t** port_ptr) {
 
 	sepol_port_t* new_port = NULL;
@@ -193,7 +251,9 @@ int sepol_port_clone(
 }
 
 /* Destroy */
-void sepol_port_free(sepol_port_t* port) {
+void sepol_port_free(
+	sepol_port_t* port) {
+
 	if (!port)
 		return;
 	
@@ -203,13 +263,27 @@ void sepol_port_free(sepol_port_t* port) {
 hidden_def(sepol_port_free)
 
 /* Context */
-sepol_context_t* sepol_port_get_con(sepol_port_t* port) {
+sepol_context_t* sepol_port_get_con(
+	const sepol_port_t* port) {
+
 	return port->con;
 }
 hidden_def(sepol_port_get_con)
 
-void sepol_port_set_con(sepol_port_t* port, sepol_context_t* con) {
+int sepol_port_set_con(
+	sepol_handle_t* handle,
+	sepol_port_t* port, 
+	sepol_context_t* con) {
+
+	sepol_context_t* newcon;
+
+	if (sepol_context_clone(handle, con, &newcon) < 0) {
+		ERR(handle, "out of memory, could not set port context");
+		return STATUS_ERR;
+	}
+
 	sepol_context_free(port->con);
-	port->con = con;
+	port->con = newcon;
+	return STATUS_SUCCESS;
 }
 hidden_def(sepol_port_set_con)
