@@ -1,3 +1,6 @@
+#ifndef _SEPOL_USER_INTERNAL_H_
+#define _SEPOL_USER_INTERNAL_H_
+
 #include <sepol/user_record.h>
 #include <sepol/users.h>
 #include "dso.h"
@@ -14,3 +17,5 @@ hidden_proto(sepol_user_key_unpack)
 hidden_proto(sepol_user_set_mlslevel)
 hidden_proto(sepol_user_set_mlsrange)
 hidden_proto(sepol_user_set_name)
+
+#endif
